// #![allow(dead_code)]

mod cpu;
mod error;
mod instructions;
mod memory;
mod types;
mod video;

use crate::cpu::Cpu;
use crate::memory::{Mem, MemMappedDevice};
use crate::video::Crtc;

use std::{
    path::PathBuf,
    sync::{Arc, RwLock},
    thread,
};

fn main() {
    let ram = MemMappedDevice::new(0x0000, 0x3FFF, 2, "ram".into());
    let control = MemMappedDevice::new(0x4000, 0x5FFF, 4, "control".into());
    let vram = MemMappedDevice::new(0x6000, 0xDFFF, 1, "vram".into());
    let rom = MemMappedDevice::new(0xE000, 0xFFFF, 4, "rom".into());
    let mut memory = Mem::new(ram);
    if let Err(error) = memory.add_area(control) {
        println!("Error adding vram: {:?}", error.desc);
    };
    if let Err(error) = memory.add_area(vram) {
        println!("Error adding vram: {:?}", error.desc);
    };
    if let Err(error) = memory.add_area(rom) {
        println!("Error adding rom: {:?}", error.desc);
    };
    let binary = match std::fs::File::open(PathBuf::from(
        "/Users/kline/projects/winter/george-emu/src/george",
    )) {
        Ok(file) => file,
        Err(error) => panic!("Couldn't open binary file! {:?}", error),
    };
    if let Err(error) = memory.read_from_bin(binary) {
        println!("{:?}", error);
    };
    memory.write(0xFFFC, 0x00).unwrap();
    memory.write(0xFFFD, 0x02).unwrap();

    let shared_memory = Arc::new(RwLock::new(memory));
    let cpu_memory = shared_memory.clone();
    let display_memory = shared_memory.clone();

    let mut screen = Crtc::new(display_memory);

    thread::spawn(move || {
        let mut cpu = Cpu::new(cpu_memory);
        cpu.reset().unwrap();
        cpu.execute();
    });
    screen.run();
}
